<dl>
    <dt>{% if field.optional %}[{% endif %}{{ field.name }}{% if field.optional %}]{% endif %}</dt>
    <dd>
        <strong>{{ field.verbose_name|title }}</strong>
        {% if field.optional %}<em>Optional</em>{% endif %}
        {% if field.doc_text %} - {{ field.doc_text }}{% endif %}
        {% if field.choices %}
        <div class="choices">
            Options:
            <ul>
                {% for choice in field.choices %}
                    <li>{{ choice }}</li>
                {% endfor %}
            </ul>
        </div>
        {% endif %}
    </dd>
</dl>
